"""Plugins for the appalerts app"""

# Projectroles dependency
from projectroles.plugins import SiteAppPluginPoint, BackendPluginPoint

from appalerts.api import AppAlertAPI
from appalerts.urls import urlpatterns


class SiteAppPlugin(SiteAppPluginPoint):
    """Site plugin for application alerts"""

    #: Name (used as plugin ID)
    name = 'appalerts'

    #: Title (used in templates)
    title = 'App Alerts'

    #: UI URLs
    urls = urlpatterns

    #: Iconify icon
    icon = 'mdi:alert-octagram'

    #: Description string
    description = 'App-specific alerts for users'

    #: Entry point URL ID
    entry_point_url_id = 'appalerts:list'

    #: Required permission for displaying the app
    app_permission = 'appalerts.view_alerts'


class BackendPlugin(BackendPluginPoint):
    """Backend plugin for application alerts"""

    #: Name (used as plugin ID)
    name = 'appalerts_backend'

    #: Title (used in templates)
    title = 'App Alerts Backend'

    #: Iconify icon
    icon = 'mdi:alert-octagram-outline'

    #: Description string
    description = 'App Alerts backend for creating and accessing alerts'

    #: URL of optional javascript file to be included
    javascript_url = 'appalerts/js/appalerts.js'

    def get_api(self, **kwargs) -> AppAlertAPI:
        """Return API entry point object"""
        return AppAlertAPI()



"""
Checkusers management command for checking user status and reporting disabled or
removed users.
"""

import ldap
import sys

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand
from django.db.models import Q, QuerySet

from projectroles.models import SODARUser
from projectroles.management.logging import ManagementCommandLogger


logger = ManagementCommandLogger(__name__)
User = get_user_model()


# Local constants
# UserAccountControl flags for disabled accounts
# TODO: Can we get these from python-ldap and print out exact status?
UAC_DISABLED_VALUES = [
    2,
    514,
    546,
    66050,
    66082,
    262658,
    262690,
    328194,
    328226,
]
UAC_LOCKED_VALUE = 16
# Messages
USER_DISABLED_MSG = 'Disabled'
USER_LOCKED_MSG = 'Locked'
USER_NOT_FOUND_MSG = 'Not found'
USER_OK_MSG = 'OK'


class Command(BaseCommand):
    help = (
        'Check user status and report disabled or removed users. Prints out '
        'user name, real name, email and status as semicolon-separated values. '
        'Disabled users can optionally be set inactive on the server.'
    )

    @classmethod
    def _print_result(cls, django_user: SODARUser, msg: str):
        print(
            f'{django_user.username};{django_user.get_full_name()};'
            f'{django_user.email};{msg}'
        )

    @classmethod
    def _get_setting_prefix(cls, primary: bool) -> str:
        return 'AUTH_LDAP{}_'.format('' if primary else '2')

    def _check_search_base_setting(self, primary: bool):
        pf = self._get_setting_prefix(primary)
        s_name = pf + 'USER_SEARCH_BASE'
        if not hasattr(settings, s_name):
            logger.error(s_name + ' not in Django settings')
            sys.exit(1)

    def _check_ldap_users(
        self,
        users: QuerySet[SODARUser],
        primary: bool,
        all_users: bool,
        deactivate: bool,
    ):
        """
        Check and print out user status for a specific LDAP server.

        :param users: QuerySet of SODARUser objects
        :param primary: Whether to check for primary or secondary server (bool)
        :param all_users: Display status for all users (bool)
        :param deactivate: Set disabled users to inactive if True (bool)
        """

        def _get_s(name):
            pf = self._get_setting_prefix(primary)
            return getattr(settings, pf + name)

        domain = _get_s('USERNAME_DOMAIN')
        domain_users = users.filter(username__endswith='@' + domain.upper())
        server_uri = _get_s('SERVER_URI')
        server_str = '{} LDAP server at "{}"'.format(
            'primary' if primary else 'secondary', server_uri
        )
        if not domain_users:
            logger.debug(f'No users found for {server_str}, skipping')
            return

        bind_dn = _get_s('BIND_DN')
        bind_pw = _get_s('BIND_PASSWORD')
        start_tls = _get_s('START_TLS')
        options = _get_s('CONNECTION_OPTIONS')
        user_filter = _get_s('USER_FILTER')
        search_base = _get_s('USER_SEARCH_BASE')

        # Enable debug if set in env
        if settings.LDAP_DEBUG:
            ldap.set_option(ldap.OPT_DEBUG_LEVEL, 255)

        # Connect to LDAP
        lc = ldap.initialize(server_uri)
        for k, v in options.items():
            lc.set_option(k, v)
        if start_tls:
            lc.protocol_version = 3
            lc.start_tls_s()
        try:
            lc.simple_bind_s(bind_dn, bind_pw)
        except Exception as ex:
            logger.error(f'Exception connecting to {server_str}: {ex}')
            return

        for d_user in domain_users:
            r = lc.search(
                search_base,
                ldap.SCOPE_SUBTREE,
                user_filter.replace('%(user)s', d_user.username.split('@')[0]),
            )
            _, l_user = lc.result(r, 60)
            if len(l_user) > 0:
                name, attrs = l_user[0]
                user_ok = True
                # logger.debug(f'Result: {name}; {attrs}')
                if (
                    'userAccountControl' in attrs
                    and len(attrs['userAccountControl']) > 0
                ):
                    val = int(attrs['userAccountControl'][0].decode('utf-8'))
                    if val in UAC_DISABLED_VALUES:
                        msg = USER_DISABLED_MSG
                        if deactivate and d_user.is_active:
                            d_user.is_active = False
                            d_user.save()
                            msg += ' (set inactive)'
                        self._print_result(d_user, msg)
                        user_ok = False
                    elif val == UAC_LOCKED_VALUE:
                        self._print_result(d_user, USER_LOCKED_MSG)
                        user_ok = False
                if all_users and user_ok:
                    self._print_result(d_user, USER_OK_MSG)
            else:  # Not found
                self._print_result(d_user, USER_NOT_FOUND_MSG)

    def add_arguments(self, parser):
        parser.add_argument(
            '-a',
            '--all',
            dest='all',
            action='store_true',
            required=False,
            help='Display results for all users even if status is OK',
        )
        parser.add_argument(
            '-d',
            '--deactivate',
            dest='deactivate',
            required=False,
            action='store_true',
            help='Set disabled users to inactive if found',
        )
        parser.add_argument(
            '-l',
            '--limit',
            dest='limit',
            required=False,
            help='Limit search to "ldap1" or "ldap2"',
        )

    def handle(self, *args, **options):
        if not settings.ENABLE_LDAP:
            logger.info('LDAP not enabled, nothing to do')
            return
        self._check_search_base_setting(primary=True)
        self._check_search_base_setting(primary=False)
        u_query = Q(
            username__endswith=f'@{settings.AUTH_LDAP_USERNAME_DOMAIN.upper()}'
        )
        if settings.ENABLE_LDAP_SECONDARY:
            q_secondary = Q(
                username__endswith=f'@'
                f'{settings.AUTH_LDAP2_USERNAME_DOMAIN.upper()}'
            )
            u_query.add(q_secondary, Q.OR)
        users = User.objects.filter(u_query).order_by('username')
        limit = options.get('limit')
        all_users = options.get('all', False)
        deactivate = options.get('deactivate', False)
        if not limit or limit == 'ldap1':
            self._check_ldap_users(users, True, all_users, deactivate)
        if settings.ENABLE_LDAP_SECONDARY and (not limit or limit == 'ldap2'):
            self._check_ldap_users(users, False, all_users, deactivate)



"""Permissions for the adminalerts app"""

import rules


# Predicates -------------------------------------------------------------


# None needed right now


# Rules ------------------------------------------------------------------


# Rules should not be needed, use permissions for user rights


# Permissions ------------------------------------------------------------


# Allow viewing alert details
rules.add_perm('adminalerts.view_alert', rules.is_authenticated)

# Allow viewing alert list
rules.add_perm('adminalerts.view_list', rules.is_superuser)

# Allow creation of alerts
rules.add_perm('adminalerts.create_alert', rules.is_superuser)

# Allow updating alerts
rules.add_perm('adminalerts.update_alert', rules.is_superuser)



"""Django checks for the projectroles app"""

from django.conf import settings
from django.core.checks import Error, Warning, register
from django.db import connection

from projectroles import app_settings
from projectroles.plugins import PluginAPI


# Local constants
W001_SETTINGS = [
    'ENABLE_LDAP',
    'ENABLE_OIDC',
    'PROJECTROLES_ALLOW_ANONYMOUS',
    'PROJECTROLES_ALLOW_LOCAL_USERS',
    'PROJECTROLES_KIOSK_MODE',
]
W001_MSG = (
    'No authentication methods enabled, only superusers can access the site. '
    'Set one or more of the following: {}'.format(', '.join(W001_SETTINGS))
)
W001 = Warning(W001_MSG, obj=settings, id='projectroles.W001')
E001_MSG = (
    'Repeated app setting definition names found in plugins. Review your '
    'plugins and ensure each app setting name is used only once within the '
    'same plugin. Affected plugin(s): {plugin_names}'
)


@register()
def check_auth_methods(app_configs, **kwargs):
    """
    Check for enabled authentication schemes. Raise error if no users other than
    superusers are able to log in with the current settings).
    """
    if not any([getattr(settings, a, False) for a in W001_SETTINGS]):
        return [W001]
    return []


@register()
def check_app_setting_defs(app_configs, **kwargs):
    """
    Check provided plugin app setting definitions to ensure the name of each
    definition is unique within its app plugin.

    This check will be skipped if a database connection is not available.
    """
    try:
        connection.ensure_connection()
    except Exception:
        return []
    plugin_api = PluginAPI()
    err_plugins = []
    for p in plugin_api.get_active_plugins():
        s_defs = p.app_settings
        if isinstance(s_defs, list):
            s_names = [d.name for d in s_defs]
            if len(set(s_names)) != len(s_names) and p.name not in err_plugins:
                err_plugins.append(p.name)
    if err_plugins:
        return [
            Error(
                E001_MSG.format(plugin_names=', '.join(err_plugins)),
                obj=app_settings,
                id='projectroles.E001',
            )
        ]
    return []



from django.urls import path

from appalerts import views, views_ajax

app_name = 'appalerts'

urlpatterns = [
    path(
        route='list',
        view=views.AppAlertListView.as_view(),
        kwargs={'status': 'active'},
        name='list',
    ),
    path(
        route='list/dismissed',
        view=views.AppAlertListView.as_view(),
        kwargs={'status': 'dismissed'},
        name='list_dismissed',
    ),
    path(
        route='redirect/<uuid:appalert>',
        view=views.AppAlertLinkRedirectView.as_view(),
        name='redirect',
    ),
    path(
        route='ajax/status',
        view=views_ajax.AppAlertStatusAjaxView.as_view(),
        name='ajax_status',
    ),
    path(
        route='ajax/dismiss/<uuid:appalert>',
        view=views_ajax.AppAlertDismissAjaxView.as_view(),
        name='ajax_dismiss',
    ),
    path(
        route='ajax/dismiss/all',
        view=views_ajax.AppAlertDismissAjaxView.as_view(),
        name='ajax_dismiss_all',
    ),
]



"""Model tests for the timeline app"""

from typing import Any, Optional, Union
from uuid import UUID

from django.forms.models import model_to_dict

from test_plus.test import TestCase

# Projectroles dependency
from projectroles.models import Project, SODARUser, SODAR_CONSTANTS
from projectroles.tests.test_models import (
    ProjectMixin,
    RoleMixin,
    RoleAssignmentMixin,
)

from timeline.models import (
    TimelineEvent,
    TimelineEventObjectRef,
    TimelineEventStatus,
    OBJ_REF_UNNAMED,
    TL_STATUS_OK,
    TL_STATUS_FAILED,
)


# SODAR constants
PROJECT_ROLE_OWNER = SODAR_CONSTANTS['PROJECT_ROLE_OWNER']
PROJECT_ROLE_DELEGATE = SODAR_CONSTANTS['PROJECT_ROLE_DELEGATE']
PROJECT_ROLE_CONTRIBUTOR = SODAR_CONSTANTS['PROJECT_ROLE_CONTRIBUTOR']
PROJECT_ROLE_GUEST = SODAR_CONSTANTS['PROJECT_ROLE_GUEST']
PROJECT_TYPE_CATEGORY = SODAR_CONSTANTS['PROJECT_TYPE_CATEGORY']
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']


# Local constants
APP_NAME_PR = 'projectroles'
EXTRA_DATA = {'test_key': 'test_val'}


class TimelineEventMixin:
    """Helper mixin for TimelineEvent creation"""

    @classmethod
    def make_event(
        cls,
        project: Optional[Project],
        app: str,
        user: Optional[SODARUser],
        event_name: str,
        description: str = '',
        classified: bool = False,
        extra_data: Union[dict, list] = {'test': 'test'},
        plugin: Optional[str] = None,
    ) -> TimelineEvent:
        """Create TimelineEvent object"""
        values = {
            'project': project,
            'app': app,
            'user': user,
            'event_name': event_name,
            'description': description,
            'classified': classified,
            'extra_data': extra_data or {},
            'plugin': plugin,
        }
        return TimelineEvent.objects.create(**values)


class TimelineEventObjectRefMixin:
    """Helper mixin for TimelineEventObjectRef creation"""

    @classmethod
    def make_object_ref(
        cls,
        event: TimelineEvent,
        obj: Any,
        label: str,
        name: str,
        uuid: Union[str, UUID],
        extra_data: Union[dict, list, None] = None,
    ) -> TimelineEventObjectRef:
        values = {
            'event': event,
            'label': label,
            'name': name,
            'object_model': obj.__class__.__name__,
            'object_uuid': uuid,
            'extra_data': extra_data or {},
        }
        """Create TimelineEventObjectRef object"""
        return TimelineEventObjectRef.objects.create(**values)


class TimelineEventStatusMixin:
    """Helper mixin for TimelineEventStatus creation"""

    @classmethod
    def make_event_status(
        cls,
        event: TimelineEvent,
        status_type: str,
        description: str = '',
        extra_data: Union[dict, list, None] = None,
    ) -> TimelineEventStatus:
        values = {
            'event': event,
            'status_type': status_type,
            'description': description,
            'extra_data': extra_data or {'test': 'test'},
        }
        """Create TimelineEventStatus object"""
        return TimelineEventStatus.objects.create(**values)


class TimelineEventTestBase(
    ProjectMixin, RoleMixin, RoleAssignmentMixin, TestCase
):
    def setUp(self):
        # Make owner user
        self.user_owner = self.make_user('owner')
        # Init roles
        self.init_roles()
        # Init project and assignment
        self.project = self.make_project(
            'TestProject', PROJECT_TYPE_PROJECT, None
        )
        self.assignment_owner = self.make_assignment(
            self.project, self.user_owner, self.role_owner
        )


class TestTimelineEvent(
    TimelineEventMixin,
    TimelineEventStatusMixin,
    TimelineEventObjectRefMixin,
    TimelineEventTestBase,
):
    def setUp(self):
        super().setUp()
        self.event = self.make_event(
            project=self.project,
            app=APP_NAME_PR,
            user=self.user_owner,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )

        self.obj_ref = self.make_object_ref(
            event=self.event,
            obj=self.assignment_owner,
            label='test_label',
            name='test_object_name',
            uuid=self.assignment_owner.sodar_uuid,
            extra_data=EXTRA_DATA,
        )

    def test_initialization(self):
        """Test TimelineEvent initialization"""
        expected = {
            'id': self.event.pk,
            'project': self.project.pk,
            'app': APP_NAME_PR,
            'plugin': None,
            'user': self.user_owner.pk,
            'event_name': 'test_event',
            'description': 'description',
            'classified': False,
            'extra_data': EXTRA_DATA,
            'sodar_uuid': self.event.sodar_uuid,
        }
        self.assertEqual(model_to_dict(self.event), expected)

    def test_initialization_no_project(self):
        """Test TimelineEvent initialization with no project"""
        self.event = self.make_event(
            project=None,
            app=APP_NAME_PR,
            user=self.user_owner,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )
        expected = {
            'id': self.event.pk,
            'project': None,
            'app': APP_NAME_PR,
            'plugin': None,
            'user': self.user_owner.pk,
            'event_name': 'test_event',
            'description': 'description',
            'classified': False,
            'extra_data': EXTRA_DATA,
            'sodar_uuid': self.event.sodar_uuid,
        }
        self.assertEqual(model_to_dict(self.event), expected)

    def test_initialization_no_user(self):
        """Test TimelineEvent initialization with no user"""
        self.event = self.make_event(
            project=self.project,
            app=APP_NAME_PR,
            user=None,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )
        expected = {
            'id': self.event.pk,
            'project': self.project.pk,
            'app': APP_NAME_PR,
            'plugin': None,
            'user': None,
            'event_name': 'test_event',
            'description': 'description',
            'classified': False,
            'extra_data': EXTRA_DATA,
            'sodar_uuid': self.event.sodar_uuid,
        }
        self.assertEqual(model_to_dict(self.event), expected)

    def test_initialization_plugin(self):
        """Test TimelineEvent initialization with specific plugin name"""
        self.event = self.make_event(
            project=None,
            app=APP_NAME_PR,
            plugin='plugin_name',
            user=self.user_owner,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )
        expected = {
            'id': self.event.pk,
            'project': None,
            'app': APP_NAME_PR,
            'plugin': 'plugin_name',
            'user': self.user_owner.pk,
            'event_name': 'test_event',
            'description': 'description',
            'classified': False,
            'extra_data': EXTRA_DATA,
            'sodar_uuid': self.event.sodar_uuid,
        }
        self.assertEqual(model_to_dict(self.event), expected)

    def test_str(self):
        """Test TimelineEvent __str__()"""
        expected = 'TestProject: test_event/owner'
        self.assertEqual(str(self.event), expected)

    def test_str_no_user(self):
        """Test TimelineEvent __str__() with no user"""
        self.event.user = None
        self.event.save()
        expected = 'TestProject: test_event'
        self.assertEqual(str(self.event), expected)

    def test_str_no_project(self):
        """Test TimelineEvent __str__() with no project"""
        self.event.project = None
        self.event.save()
        expected = 'test_event/owner'
        self.assertEqual(str(self.event), expected)

    def test_str_no_project_no_user(self):
        """Test TimelineEvent __str__() with no project or user"""
        self.event.project = None
        self.event.user = None
        self.event.save()
        expected = 'test_event'
        self.assertEqual(str(self.event), expected)

    def test_repr(self):
        """Test TimelineEvent __repr__()"""
        expected = "TimelineEvent('TestProject', 'test_event', 'owner')"
        self.assertEqual(repr(self.event), expected)

    def test_repr_no_user(self):
        """Test TimelineEvent __repr__() with no user"""
        self.event.user = None
        self.event.save()
        expected = "TimelineEvent('TestProject', 'test_event', 'N/A')"
        self.assertEqual(repr(self.event), expected)

    def test_repr_no_project(self):
        """Test TimelineEvent __repr__() with no project"""
        self.event.project = None
        self.event.save()
        expected = "TimelineEvent('N/A', 'test_event', 'owner')"
        self.assertEqual(repr(self.event), expected)

    def test_repr_no_project_no_user(self):
        """Test TimelineEvent __repr__() with no project or user"""
        self.event.project = None
        self.event.user = None
        self.event.save()
        expected = "TimelineEvent('N/A', 'test_event', 'N/A')"
        self.assertEqual(repr(self.event), expected)

    def test_add_object(self):
        """Test add_object()"""
        # Init new user and role
        new_user = self.make_user('new_user')
        new_as = self.make_assignment(
            self.project, new_user, self.role_delegate
        )
        new_obj = self.event.add_object(
            obj=new_as, label='new_label', name='new_name'
        )
        expected = {
            'id': new_obj.pk,
            'event': self.event.pk,
            'label': 'new_label',
            'name': 'new_name',
            'object_uuid': new_as.sodar_uuid,
            'object_model': 'RoleAssignment',
            'extra_data': {},
            'sodar_uuid': new_obj.sodar_uuid,
        }
        self.assertEqual(model_to_dict(new_obj), expected)

    def test_add_object_no_name(self):
        """Test add_object() with no name for object"""
        new_user = self.make_user('new_user')
        new_user.name = 'New User'
        new_as = self.make_assignment(
            self.project, new_user, self.role_delegate
        )
        new_obj = self.event.add_object(obj=new_as, label='new_label', name='')
        self.assertEqual(new_obj.name, OBJ_REF_UNNAMED)

    def test_find_event_name(self):
        """Test TimelineEvent.find() with event name"""
        objects = TimelineEvent.objects.find(['test_event'])
        self.assertEqual(len(objects), 1)
        self.assertEqual(objects[0], self.event)

    def test_find_description(self):
        """Test TimelineEvent.find() with event description"""
        objects = TimelineEvent.objects.find(['description'])
        self.assertEqual(len(objects), 1)
        self.assertEqual(objects[0], self.event)

    def test_find_object(self):
        """Test TimelineEvent.find() with object reference"""
        objects = TimelineEvent.objects.find(['test_object_name'])
        self.assertEqual(len(objects), 1)
        self.assertEqual(objects[0], self.event)

    def test_find_fail(self):
        """Test TimelineEvent.find() with no matches"""
        objects = TimelineEvent.objects.find(['asdfasdfafasdf'])
        self.assertEqual(len(objects), 0)

    def test_find_user_username(self):
        """Test TimelineEvent.find() with user username"""
        objects = TimelineEvent.objects.find([self.user_owner.username])
        self.assertEqual(len(objects), 1)
        self.assertEqual(objects[0], self.event)

    def test_find_user_username_not_found(self):
        """Test TimelineEvent.find() with user username not found"""
        user_new = self.make_user('user_new')
        objects = TimelineEvent.objects.find([user_new.username])
        self.assertEqual(len(objects), 0)

    def test_find_user_full_name(self):
        """Test TimelineEvent.find() with user full name"""
        self.user_owner.name = 'Owner User'
        self.user_owner.save()
        objects = TimelineEvent.objects.find([self.user_owner.name])
        self.assertEqual(len(objects), 1)
        self.assertEqual(objects[0], self.event)

    def test_find_user_full_name_not_found(self):
        """Test TimelineEvent.find() with user full name not found"""
        self.user_owner.name = 'Owner User'
        self.user_owner.save()
        objects = TimelineEvent.objects.find(['Other User'])
        self.assertEqual(len(objects), 0)


class TestTimelineEventObjectRef(
    TimelineEventMixin, TimelineEventObjectRefMixin, TimelineEventTestBase
):
    def setUp(self):
        super().setUp()
        self.event = self.make_event(
            project=self.project,
            app=APP_NAME_PR,
            user=self.user_owner,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )
        self.obj_ref = self.make_object_ref(
            event=self.event,
            obj=self.assignment_owner,
            label='test_label',
            name='test_name',
            uuid=self.assignment_owner.sodar_uuid,
            extra_data=EXTRA_DATA,
        )

    def test_initialization(self):
        """Test TimelineEventObjectRef initialization"""
        expected = {
            'id': self.obj_ref.pk,
            'event': self.event.pk,
            'label': 'test_label',
            'name': 'test_name',
            'object_model': 'RoleAssignment',
            'object_uuid': self.assignment_owner.sodar_uuid,
            'extra_data': EXTRA_DATA,
            'sodar_uuid': self.obj_ref.sodar_uuid,
        }
        self.assertEqual(model_to_dict(self.obj_ref), expected)

    def test__str__(self):
        """Test TimelineEventObjectRef __str__()"""
        expected = 'TestProject: test_event/owner (test_name)'
        self.assertEqual(str(self.obj_ref), expected)

    def test__repr__(self):
        """Test TimelineEventObjectRef __repr__()"""
        expected = (
            "TimelineEventObjectRef('TestProject', 'test_event', "
            "'owner', 'test_name')"
        )
        self.assertEqual(repr(self.obj_ref), expected)

    def test_get_object_events(self):
        """Test get_object_events() in TimelineEventManager"""
        events = TimelineEvent.objects.get_object_events(
            project=self.project,
            object_model=self.obj_ref.object_model,
            object_uuid=self.obj_ref.object_uuid,
        )
        self.assertEqual(events.count(), 1)
        self.assertEqual(events[0], self.event)


class TestTimelineEventStatus(
    TimelineEventMixin, TimelineEventStatusMixin, TimelineEventTestBase
):
    def setUp(self):
        super().setUp()
        self.event = self.make_event(
            project=self.project,
            app=APP_NAME_PR,
            user=self.user_owner,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )
        self.event_status_submit = self.make_event_status(
            event=self.event,
            status_type='SUBMIT',
            description='SUBMIT',
            extra_data=EXTRA_DATA,
        )
        self.event_status_ok = self.make_event_status(
            event=self.event,
            status_type=TL_STATUS_OK,
            description=TL_STATUS_OK,
            extra_data=EXTRA_DATA,
        )

    def test_initialization(self):
        """Test TimelineEventStatus init"""
        expected = {
            'id': self.event_status_ok.pk,
            'sodar_uuid': self.event_status_ok.sodar_uuid,
            'event': self.event.pk,
            'status_type': TL_STATUS_OK,
            'description': TL_STATUS_OK,
            'extra_data': EXTRA_DATA,
        }
        self.assertEqual(model_to_dict(self.event_status_ok), expected)

    def test_initialization_no_user(self):
        """Test TimelineEventStatus without user"""
        expected = {
            'id': self.event_status_ok.pk,
            'sodar_uuid': self.event_status_ok.sodar_uuid,
            'event': self.event.pk,
            'status_type': TL_STATUS_OK,
            'description': TL_STATUS_OK,
            'extra_data': EXTRA_DATA,
        }
        self.event = self.make_event(
            project=self.project,
            app=APP_NAME_PR,
            user=None,
            event_name='test_event',
            description='description',
            classified=False,
            extra_data=EXTRA_DATA,
        )
        self.assertEqual(model_to_dict(self.event_status_ok), expected)

    def test__str__(self):
        """Test TimelineEventStatus __str__()"""
        expected = f'TestProject: test_event/owner ({TL_STATUS_OK})'
        self.assertEqual(str(self.event_status_ok), expected)

    def test__str__no_user(self):
        """Test __str__() with no user"""
        self.event.user = None
        self.event.save()
        expected = f'TestProject: test_event ({TL_STATUS_OK})'
        self.assertEqual(str(self.event_status_ok), expected)

    def test__repr__(self):
        """Test TimelineEventStatus __repr__()"""
        expected = f"TimelineEventStatus('TestProject', 'test_event', 'owner', '{TL_STATUS_OK}')"
        self.assertEqual(repr(self.event_status_ok), expected)

    def test__repr__no_user(self):
        """Test __repr__() with no user"""
        self.event.user = None
        self.event.save()
        expected = f"TimelineEventStatus('TestProject', 'test_event', 'N/A', '{TL_STATUS_OK}')"
        self.assertEqual(repr(self.event_status_ok), expected)

    def test_get_status(self):
        """Test TimelineEventStatus get_status()"""
        status = self.event.get_status()
        expected = {
            'id': status.pk,
            'sodar_uuid': self.event_status_ok.sodar_uuid,
            'event': self.event.pk,
            'status_type': TL_STATUS_OK,
            'description': TL_STATUS_OK,
            'extra_data': EXTRA_DATA,
        }
        self.assertEqual(model_to_dict(status), expected)

    def test_get_timestamp(self):
        """Test TimelineEventStatus get_timestamp()"""
        timestamp = self.event.get_timestamp()
        self.assertEqual(timestamp, self.event_status_ok.timestamp)

    def test_get_status_changes(self):
        """Test TimelineEventStatus get_status_changes()"""
        status_changes = self.event.get_status_changes()
        self.assertEqual(status_changes.count(), 2)
        self.assertEqual(status_changes[0], self.event_status_submit)

    def test_get_status_changes_reverse(self):
        """Test the get_status_changes() function of TimelineEvent with
        reverse=True"""
        status_changes = self.event.get_status_changes(reverse=True)
        self.assertEqual(status_changes.count(), 2)
        self.assertEqual(status_changes[0], self.event_status_ok)

    def test_set_status(self):
        """Test TimelineEventStatus set_status()"""
        new_status = self.event.set_status(
            TL_STATUS_FAILED,
            status_desc=TL_STATUS_FAILED,
            extra_data=EXTRA_DATA,
        )
        expected = {
            'id': new_status.pk,
            'sodar_uuid': new_status.sodar_uuid,
            'event': self.event.pk,
            'status_type': TL_STATUS_FAILED,
            'description': TL_STATUS_FAILED,
            'extra_data': EXTRA_DATA,
        }
        self.assertEqual(model_to_dict(new_status), expected)



# Generated by Django 4.2.18 on 2025-02-04 09:39

from django.db import migrations


def rename_plugin(apps, schema_editor):
    Plugin = apps.get_model('djangoplugins', 'Plugin')
    plugin = Plugin.objects.filter(name='tokens').first()
    if plugin:
        plugin.pythonpath = 'tokens.plugins.SiteAppPlugin'
        plugin.save()


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.RunPython(
            code=rename_plugin,
            reverse_code=migrations.RunPython.noop,
        )
    ]



"""Syncgroups management command for synchronizing user groups"""

from django.contrib import auth
from django.core.management.base import BaseCommand

from projectroles.management.logging import ManagementCommandLogger


logger = ManagementCommandLogger(__name__)
User = auth.get_user_model()


class Command(BaseCommand):
    help = 'Synchronizes user groups based on user name'

    def handle(self, *args, **options):
        logger.info('Synchronizing user groups..')
        for user in User.objects.all().order_by('username'):
            user.groups.clear()
            user.save()  # Group is updated during save
            if user.groups.count() > 0:
                logger.info(
                    f'Group set: {user.username} -> {user.groups.first().name}'
                )
        logger.info(f'Synchronized groups for {User.objects.count()} users')



"""Serializers for the sodarcache app REST API"""

from rest_framework import serializers

# Projectroles dependency
from projectroles.serializers import SODARProjectModelSerializer

from sodarcache.models import JSONCacheItem


class JSONCacheItemSerializer(SODARProjectModelSerializer):
    """Serializer for the JSONCacheItem model"""

    user = serializers.SlugRelatedField(slug_field='sodar_uuid', read_only=True)

    class Meta:
        model = JSONCacheItem
        fields = [
            'project',
            'app_name',
            'name',
            'date_modified',
            'user',
            'data',
        ]  # NOTE: No UUID returned as these should not be referred to directly
        read_only_fields = ['date_modified']
